/* DBIVBIN.C 
   Template for QC 2.5 routines intended for conversion by MAKEBIN.EXE
   to BINs suitable for use with dBASE IV. The authorship of MAKEBIN by
   Michael Day is gratefully acknowledged.

   This template has 2 uses:

   1. With Make Option set to debug
      it creates a standalone .EXE file, that is easily debugged.
      Set test parameters below.

   2. With Make Option set to Release
      it creates an .EXE file that can then be converted by MAKEBIN.EXE:

      MAKEBIN dbivbin.exe

   This template has been tested only with QC 2.5, small memory model.
*/

/* Common code */
extern unsigned int _near _cdecl _psp;

/* define NULL pointer value */
#define NULL    ((void *)0)

#define MK_FARPTR(hi,low) (*((unsigned long _far *)(((unsigned long)(hi) \
                          << 16) | (unsigned)(low))))

struct ipc_struct
{
    char bipc[4];  /* ipc->bipc contains 'BIPC' if the BIN properly loaded */
    unsigned int oldflgs;   /* flags on entry to the BIN file */
    unsigned int oldax;     /* AX register on entry to the BIN file */
    unsigned int oldbx;     /* BX register on entry to the BIN file */
    unsigned int oldcx;     /* CX register on entry to the BIN file */
    unsigned int olddx;     /* DX register on entry to the BIN file */
    unsigned int oldsi;     /* SI register on entry to the BIN file */
    unsigned int olddi;     /* DI register on entry to the BIN file */
    unsigned int oldds;     /* DS register on entry to the BIN file */
    unsigned int oldes;     /* ES register on entry to the BIN file */
    unsigned int oldbp;     /* BP register on entry to the BIN file */
    unsigned long OldStk;   /* SS:SP registers, pointer to stack on entry  */
    unsigned long WrkPtr;   /* CS:IP, pointer to current Entry address */
    unsigned long WrkStk;   /*  pointer to stack on exit */
    unsigned int BinDS;     /* DS seg on exit from BIN file */
    unsigned int BinSS;     /* SS seg on exit from BIN file */
    unsigned long LoadPtr;  /* pointer to original entry point of BIN */
    unsigned long LoadStk;  /* pointer to original stack location of BIN  */
    unsigned int PspSeg;    /* current PSP seg of BIN file */
    unsigned int OldPsp;    /* caller's PSP segment */
    unsigned int PrgSiz;    /* Total size of bin file in paragraphs */
};

struct ipc_struct _far * ipc; /* pointer to interface structure */
int argc;                     /* number of parameters */
char _far * argv[8];          /* array of far pointers to parameters */
struct ipc_struct _far * ipcinit( void );

#ifndef NDEBUG    /* this code for debugging */

/* debug function prototypes */
void savestate( void );
void verifystate( void );

struct ipc_struct ipcdebug;   /* dummy ipc structure */
unsigned int argvlengths[8];  /* array of parameter lengths */

/*
 TEST parameters
 set these  as appropriate for debugging
*/
argc = 2;                                        /* number of parameters */
char _far *p1 = "Z";                           /* 1'st parameter       */
char _far *p2 = "BARqsw.DBF";                       /* etc.                 */
char _far *p3 = "";
char _far *p4 = "";
char _far *p5 = "";
char _far *p6 = "";
char _far *p7 = "";
/**/
#endif


/*
*** Actual routine(s) go here ********************************************
*/

/*

BIN to deactivate/activate DB4 production MDX. Requires 2 parameters,
1'st is either capital 'Z' or 'U', 2'nd is database file name.
'Z' as 1'st parameter will write 0h to offset 28 of the DBF's header,
deactivating any production MDX. 'U' as 1'st parameter will write 1h
to offset 28 of the DBF's header, activating any production MDX.
Syntax:
LOAD mdxzap
lc_result=CALL("mdxzap","Z","myfile.dbf")
The CALL function will return the following codes:
'0' == Success
'1' == Some problem with the BIN itself.
'2' == Wrong number of parameters
'3' == Parameter 1 not 'Z' or 'U'
'4' == Could not open DBF
'5' == Could not write to DBF

*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <string.h>

void main( void )
{
	/* variable declarations */
    int fh;
    char byte_to_write;
	char dbf_name[50];
    char parm1[2];
    char far *strpos;
    int count;


    if( ipcinit() == NULL ) /* NULL = invalid interface record */
        *argv[1] = '1';
    else
    {

#ifndef NDEBUG    /* store parameter lengths if debugging */
		savestate();
#endif
		if(argc != 2)
            *argv[1] = '2';
		else
		{
            /* copy argv[2] to near string */
            count = 0;
            for(strpos = argv[2]; *strpos != 0; strpos++)
			{
                dbf_name[count] = *strpos;
                ++count;
			}
            dbf_name[count] = 0;

            /* copy argv[1] to near string */
            parm1[0] = *argv[1];
            parm1[1] = 0;

            /* validate parameter 1 */
            switch(*parm1)
            {
            case 'Z':
                byte_to_write = 0;
                break;
            case 'U':
                byte_to_write = 1;
                break;
            default:
                *argv[1] = '3';
            }

            if (*argv[1] != '3')
            {
                /* open and write DBF file */
                if( (fh = open(dbf_name, O_BINARY|O_WRONLY)) == -1 )
                    *argv[1] = '4';
                else
                {
                    if (lseek(fh,28L,SEEK_SET) != 28L)
						*argv[1] = '5';
                    else
                    {
                        if (write(fh,&byte_to_write,1) != 1)
                            *argv[1] = '5';
                        else
                            *argv[1] = '0';
                    }
                    close(fh);
                }
            }
		}
	}

#ifndef NDEBUG    /* check & print parameters if debugging */
	verifystate();
#endif

}

/*
*** End of routine ********************************************************
*/

#ifndef NDEBUG    /* this code for debugging */

/* initialize dummy ipc structure for debugging */
struct ipc_struct _far * ipcinit( void )
{
    ipc = &ipcdebug;
    ipc->bipc[0] = 'B';
    ipc->bipc[1] = 'I';
    ipc->bipc[2] = 'P';
    ipc->bipc[3] = 'C';
    argv[1] = p1;
    argv[2] = p2;
    argv[3] = p3;
    argv[4] = p4;
    argv[5] = p5;
    argv[6] = p6;
    argv[7] = p7;
    return(ipc);
}

/* save parameter lengths */
void savestate( void )
{
    int i;
    for( i = 1; i <= argc; i++)
        argvlengths[i] = _fstrlen(argv[i]);
}

/* check parameter lengths and print them */
void verifystate( void )
{
    int i;
    for( i = 1; i <= argc; i++)
    {
        if( _fstrlen(argv[i]) != argvlengths[i] )
            printf( "\nWarning! Parm%d's length has changed from %d to %d!", \
                    i, argvlengths[i], _fstrlen(argv[i]) );
        printf( "\nParm%d: %s", i, argv[i] );
    }
}

#else

/* real ipcinit function */
struct ipc_struct _far * ipcinit( void )
{
    int parm_offset, i;
    ipc = (struct ipc_struct _far *)MK_FARPTR(_psp-2,0);
    if (ipc->bipc[0] == 'B' && ipc->bipc[1] == 'I' &&\
        ipc->bipc[2] == 'P' && ipc->bipc[3] == 'C')
    {
        argc = ipc->oldcx; /* dBASE4 stores the number of parms in CX */
        parm_offset = 0;
        for( i = 1; i <= 7; i++)
        {
            /* dBASE4 sets ES:DI to point to an array
               of far pointers to parameters.         */
            argv[i] = (char _far *)MK_FARPTR(ipc->oldes,ipc->olddi+parm_offset);
            parm_offset += 4;
        }
        return(ipc);
    }
    else
        return(NULL);       /* NULL = invalid interface record */
}
#endif
